package com.maverick.sshd.sftp;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import com.maverick.sshd.Connection;
import com.maverick.sshd.SftpFileAttributes;
import com.maverick.sshd.platform.PermissionDeniedException;
import com.maverick.util.UnsignedInteger64;

public class DirectFile extends AbstractDirectFile<DirectFile> {
	
	public DirectFile(String path, AbstractFileFactory<DirectFile> fileFactory, Connection con, String homeDir) throws IOException {
		super(path, fileFactory, con, homeDir);
	}

	public SftpFileAttributes getAttributes() throws FileNotFoundException {
		
		if(!f.exists()) {
			throw new FileNotFoundException();
		}
		
		SftpFileAttributes attrs = new SftpFileAttributes("UTF-8");
		attrs.setTimes(new UnsignedInteger64(f.lastModified() / 1000), 
				new UnsignedInteger64(f.lastModified() / 1000));
		
		attrs.setPermissions(String.format("%s%s-------", (isReadable() ? "r"
				: "-"), (isWritable() ? "w" : "-")));
		
		if(f.isDirectory()) {
			attrs.setType(SftpFileAttributes.SSH_FILEXFER_TYPE_DIRECTORY);
		} else if(f.exists()) {
			attrs.setType(SftpFileAttributes.SSH_FILEXFER_TYPE_REGULAR);
		} else {
			attrs.setType(SftpFileAttributes.SSH_FILEXFER_TYPE_UNKNOWN);
		}
		
		if(!isDirectory()) {
			attrs.setSize(new UnsignedInteger64(f.length()));
		}
		  
	    return attrs;
	}

	public List<AbstractFile> getChildren() throws IOException {
		
		File[] files = f.listFiles();
		List<AbstractFile> files2 = new ArrayList<AbstractFile>();
		for(File f : files) {
			files2.add(new DirectFile(f.getAbsolutePath(), fileFactory, con, homeDir));
		}
		return files2;
	}

	public AbstractFile resolveFile(String child) throws IOException,
			PermissionDeniedException {
		return new DirectFile(new File(f, child).getAbsolutePath(), fileFactory, con, homeDir);
	}
}
