package com.maverick.sshd.sftp;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.Constructor;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.maverick.events.Event;
import com.maverick.sshd.Connection;
import com.maverick.sshd.platform.PermissionDeniedException;

@SuppressWarnings({ "unchecked", "static-access" })
public class DirectFileFactory extends AbstractDirectFileFactory<DirectFile> {

	static Logger log = LoggerFactory.getLogger(DirectFileFactory.class);
	
	File defaultPath = new File(".");
	static Class<DirectFile> clz = null;
	static Constructor<DirectFile> constructor = null;
	static {
		
		try {
			clz = (Class<DirectFile>)DirectFileFactory.class.forName("com.maverick.sshd.sftp.DirectFileJava7");
			constructor = clz.getConstructor(String.class, AbstractFileFactory.class, Connection.class, String.class);
		} catch (Throwable e) {
			log.warn("Falling back to simple DirectFile implementation as current version of Java does not appear to support Path and FileAttributes APIs");
		}
	}
	public DirectFileFactory() {
		super(new DirectFileHomeFactory());
	}
	
	public DirectFileFactory(AbstractFileHomeFactory homeFactory) {
		super(homeFactory);
	}
	
	public DirectFile getFile(String path, Connection con)
			throws PermissionDeniedException, IOException {
		
		if(constructor!=null) {
			try {
				return constructor.newInstance(path, this, con, homeFactory.getHomeDirectory(con));
			} catch (Throwable e) {
				e.printStackTrace();
			}
		} 
		
		return new DirectFile(path, this, con, homeFactory.getHomeDirectory(con));
		
	}

	public Event populateEvent(Event evt) {
		return evt;
	}

}
