package com.maverick.sshd.sftp;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.nio.file.FileSystems;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.attribute.BasicFileAttributes;
import java.nio.file.attribute.DosFileAttributes;
import java.nio.file.attribute.PosixFileAttributes;
import java.nio.file.attribute.PosixFilePermissions;
import java.util.ArrayList;
import java.util.List;

import com.maverick.sshd.Connection;
import com.maverick.sshd.SftpFileAttributes;
import com.maverick.sshd.platform.PermissionDeniedException;
import com.maverick.util.UnsignedInteger32;
import com.maverick.util.UnsignedInteger64;

public class DirectFileJava7 extends DirectFile {
	
	public DirectFileJava7(String path, AbstractFileFactory<DirectFile> fileFactory, Connection con, String homeDir) throws IOException {
		super(path, fileFactory, con, homeDir);
	}

	public SftpFileAttributes getAttributes() throws FileNotFoundException {
		
		if(!f.exists())
			throw new FileNotFoundException();
		
		SftpFileAttributes attrs = new SftpFileAttributes("UTF-8");
		Path file = FileSystems.getDefault().getPath(f.getAbsolutePath());
		
		try {
		
			BasicFileAttributes attr = Files.readAttributes(file, BasicFileAttributes.class);
			
			attrs.setTimes(new UnsignedInteger64(attr.lastAccessTime().toMillis() / 1000), 
					new UnsignedInteger64(attr.lastModifiedTime().toMillis() / 1000));

			if(attr.isDirectory())
				attrs.setType(SftpFileAttributes.SSH_FILEXFER_TYPE_DIRECTORY);
			if(attr.isRegularFile())
				attrs.setType(SftpFileAttributes.SSH_FILEXFER_TYPE_REGULAR);
			if(attr.isSymbolicLink())
				attrs.setType(SftpFileAttributes.SSH_FILEXFER_TYPE_SYMLINK);
			if(attr.isOther())
				attrs.setType(SftpFileAttributes.SSH_FILEXFER_TYPE_SPECIAL);
			
			attrs.setSize(new UnsignedInteger64(attr.size()));

			
		} catch (UnsupportedOperationException | IOException e) {
		}
		
		try {
			PosixFileAttributes attr =  Files.readAttributes(file, PosixFileAttributes.class);
			
			attrs.setGID(new UnsignedInteger32(attr.group().hashCode()), attr.group().getName());
			attrs.setUID(new UnsignedInteger32(attr.owner().hashCode()), attr.owner().getName());
			
			attrs.setPermissions(PosixFilePermissions.toString(attr.permissions()));
			
			hidden = f.getName().startsWith(".");

			// We return now as we have enough information
			return attrs;
			
		} catch (UnsupportedOperationException | IOException e) {
		}

		
		try {
			DosFileAttributes attr = Files.readAttributes(file,
						DosFileAttributes.class);
		
			hidden = attr.isHidden();
		
			String read = "r";
			String write = attr.isReadOnly() ? "-" : "w";
			String exe = (f.getName().endsWith(".exe") 
					|| f.getName().endsWith(".com") 
					|| f.getName().endsWith(".cmd")) ? "x" : "-";
			
			attrs.setPermissions(read + write + exe + read + write + exe + read + write + exe);
	
		} catch(UnsupportedOperationException | IOException e) {
		}
		  
	    return attrs;
	}

	public List<AbstractFile> getChildren() throws IOException {
		
		File[] files = f.listFiles();
		List<AbstractFile> files2 = new ArrayList<AbstractFile>();
		for(File f : files) {
			files2.add(new DirectFileJava7(f.getAbsolutePath(), fileFactory, con, homeDir));
		}
		return files2;
	}

	public AbstractFile resolveFile(String child) throws IOException,
			PermissionDeniedException {
		return new DirectFileJava7(new File(f, child).getAbsolutePath(), fileFactory, con, homeDir);
	}
}
